<?php

namespace Aerex\TaskwarriorPlugin\TW;
use Zend\Validator\Uuid;
use Carbon\Carbon;
use Sabre\VObject\Component\VTodo;

/**
 * Task
 * @author Aerex
 */
class Task { 

/**
     * @var string
     *
     * @JMS\Type("string")
     */
    private $uuid;

    /**
     * @var string
     *
     * @JMS\Type("string")
     */
    private $description;

    /**
     * @var string
     *
     */
    private $priority;

    /**
     * @var string
     *
     */
    private $project;

    /**
     * @var Carbon
     *
     */
    private $due;

    /**
     * @var Carbon
     *
     */
    private $wait;

    /**
     * @var array
     *
     */
    private $tags;

    /**
     * @var float
     *
     */
    private $urgency;

    /**
     * @var Carbon
     *
     */
    private $entry;

    /**
     * @var Carbon
     *
     */
    private $start;

    /**
     * @var string
     *
     */
    private $recur;

    /**
     * @var Carbon
     *
     */
    private $until;

    /**
     * @var Annotation[]
     *
     */
    private $annotations = [];

    /**
     * @var Carbon
     *
     */
    private $modified;

    /**
     * @var Carbon
     *
     */
    private $end;

    /**
     * @var string
     *
     */
    private $status;

    /**
     * @var Task[]|ArrayCollection
     *
     */
    private $depends;


  public function __construct($UUID=null){
    $validator = new Uuid();

    if(!isset($UUID)){
      $this->uuid = uniqid();
    } else if(isset($UUID) && !$validator->isValid($UUID)){
      throw new Exception(sprintf('%s is not a valid uuid', $UUID));
    } 

    $this->uuid = $UUID;


  }

    public function getUuid(){
      return $this->uuid;

    }

    public function convertToArray(){
      $tagsString = null;
      $dependsString = null;
      $dueString = null;

      // Process array properties
      if(isset($this->tags)){
        $tagsString = implode(',', $this->tags);
      }
      if(isset($this->depends)){
        $dependsString = implode(',', $this->depends);
      }

      // Process date properties
      if(isset($this->due)){
        $dueString = $this->due->format('Y-m-dTH:i:s');

      }
      


      return array(
        "status" => $this->status,
        "start" => $this->start,
        "wait" => $this->wait,
        "end" => $this->end,
        "entry" => $this->entry,
        "priority" => $this->priority,
        "project" => $this->project,
        "due" => $this->due,
        "tags" => $tagsString,
        "urgency" => $this->urgency,
        "recu" => $this->recur,
        "until" => $this->until,
        "tags" => $this->annotations,
        "modified" => $this->modified,
        "depends" => $dependsString
      );
    }

/**
 *
 *
 * If description  is not available attempt to summary, otherwise throw Exception
 */
public function setDescription(VTodo $component){
  if(!isset($component->DESCRIPTION) && isset($component->SUMMARY)){
    $this->description = $component->SUMMARY;
  } else if(!isset($component->DESCRIPTION) && !isset($component->SUMMARY)){
    throw new Exception("Task must have a description or summary");
  } else {
    $this->description = $component->DESCRIPTION;
  }

}

public function getDescription(){
  return $this->description;
}

public function setEntryTime(VTodo $document){

  if(isset($document->DTSTAMP)){
    $this->entry =  new Carbon($document->DTSTAMP->getDateTime()->format(\DateTime::W3C));
  } else {
    throw new Exception('Task must have a entry time');
  }

}

public function getEntryTime(){
  return $this->entry;
}

public function setStartTime(VTodo $document){
  if(isset($document->DTSTART)){
    $this->start = new Carbon($document->DTSTART->getDateTime()->format(\DateTime::W3C));
  }
}

public function getStartTime(){
  return $this->start;

}

public function setModifiedTime(VTodo $document){
    if(isset($document->{'LAST-MODIFIED'})){
      $this->modified = new Carbon($document->{'LAST-MODIFIED'}->getDateTime()->format(\DateTime::W3C));
    }
}

public function getModifiedTime(){
  return $this->modified;
}

public function setDue(VTodo $document){

  if(isset($document->DUE)){
    $this->due = new Carbon($document->DUE->getDateTime()->format(\DateTime::W3C));
  }
}
public function getDue($format=null){
  if($format != null){
    return $this->due->format($format);
  }

  return $this->due;
}

public function setStopTime(VTodo $document){
  if(isset($document->DTEND)){
    $this->end = new Carbon($document->DTEND->getDateTime()->format(\DateTime::W3C));
  }
}

public function getStopTime(){
  return $this->end;
}

public function setCategories(VTodo $document){
  if(isset($document->CATEGORIES)){  
    $this->tags = explode(',', (string)$document->CATEGORIES);
  }
}

public function getCategories(){
  return $this->tags;
}

public function setStatus(VTodo $document){
  if(isset($document->STATUS)){
    switch((string)$document->STATUS){
      case 'NEEDS-ACTION':
        $this->status = 'pending';
        break;
      case 'COMPLETED':
        $this->status = 'completed';
        break;
      case 'CANCELED':
        $this->status = 'deleted';
        break;
    } 
  }
}

public function getRecurrence(){
  return $this->recur;

}
public function setRecurrence(VTodo $document){
  if(isset($document->RRULE)){
      $this->recur = $document->RRULE;
  }
}

public function getStatus(){
  return $this->status;
}





}

